open Ast_program
open Format
open Unix

let incr_env m =
  SMap.map (fun x -> x + 1) m

exception Not_closed of string

module IMap = Map.Make(struct
    type t = int
    let compare = Pervasives.compare
  end)

let v =
  let r = ref 0 in
  let f () = incr r; !r in
  f

let find =
  let r = ref 0 in
  let f x chan =
    try chan, SMap.find x chan
    with Not_found
      -> (let v = !r in
          incr r;
          (SMap.add x v chan, v)) in
  f

(**
 * mortar: converts pprogs into progs
*)
let rec mortar chan var = function
  | PSend (a,p) -> let (chan,a) = find a chan in
    let (chan, res) = mortar chan var p in
    (chan, [Send (a, res)])
  | PRec (a,x,p) -> let (chan,a) = find a chan in
    let var = SMap.add x 0 (incr_env var) in
    let (chan, res) = mortar chan var p in
    (chan, [Recv (a, res)])
  | PComp (p,q) -> let (chan, res_p) = mortar chan var p in
    let (chan, res_q) = mortar chan var q in
    (chan, res_p @ res_q)
  | PExe x ->
    (chan, [Var (try SMap.find x var
                 with Not_found -> raise (Not_closed x))])
  | PZero -> (chan, [])

(**
 * compile: compiles a prog into a pseudo-process understandable by the machine
*)
let compile name p =
  let fd = openfile ((Filename.remove_extension name)^"_prep.shoc")
      [O_CREAT; O_TRUNC; O_WRONLY] 0o755 in
  let oc = out_channel_of_descr fd in
  set_binary_mode_out oc false;
  let fmt = formatter_of_out_channel oc in
  let (chan,p) = mortar SMap.empty SMap.empty p in
  let rec compile_one top fmt = function
    | Send (a,p) -> fprintf fmt "%s \
Status<B_var.B_recv.B_send! ACK_status<>> | \
Channel<%d> | \
Program<%a>@?" (if top then "Environment<EMPTY> |" else "")
                      a (compile_list false true) p
    | Recv (a,p) -> fprintf fmt "%s \
Status<B_var.B_send.B_recv! ACK_status<>> | \
Channel<%d> | \
Program<%a>@?" (if top then "Environment<EMPTY> |" else "")
                      a (compile_list false true) p
    | Var x -> fprintf fmt "%s \
Status<B_send.B_recv.B_var! ACK_status<>> | \
Channel<0> | Program<%d>@?" (if top then "Environment<EMPTY> |" else "") x
  and compile_list top b fmt = function
    | a::l -> fprintf fmt "%s%a%a" (if b then "[" else "")
                (compile_one top) a
                (if l = [] then (fun fmt _ -> fprintf fmt "]@?")
                 else (fun fmt _ -> fprintf fmt " ; %a"
                          (compile_list top false) l)) ()
    | [] -> fprintf fmt "EMPTY@?" in
  let i_chan = SMap.fold (fun a n p -> IMap.add n a p) chan IMap.empty in
  let chan = List.fold_left (fun l (_,a) -> a::l) [] (IMap.bindings i_chan) in
  let () = match chan with
    | [] ->  ()
    | last::l -> let chan = List.rev l in
      let compile_chan fmt a =
        List.iter (fun x -> fprintf fmt "Message(x).%s<x> ;@." x) a in
      fprintf fmt "Channel_name<[@.%a%s@.]> | @." compile_chan chan
        ("Message(x)."^last^"<x>") in
  fprintf fmt "@.Ref_list<@.%a@.>@.@." (compile_list true true) p;
  close_out oc
