open Parser_sugar
open Lexer_sugar

open Format
open Arg
open Lexing
open Unix

let usage = "usage: dshoc file"

let file =
  let f = ref None in
  let set_file s =
    if not (Filename.check_suffix s ".shoc")
    then raise (Arg.Bad "no .shoc extension");
    f := Some s in
  Arg.parse [] set_file usage;
  match !f with
  | None -> Arg.usage [] usage; exit 1
  | Some f -> f

let reportPos (b,e) =
  let l = b.pos_lnum in
  let fc = b.pos_cnum - b.pos_bol + 1 in
  let lc = e.pos_cnum - b.pos_bol + 1 in
  eprintf "File \"%s\", line %d, characters %d-%d:\n@." file l fc lc


let () =
  let c = open_in file in
  let lb = Lexing.from_channel c in
  try
    let f = Parser_sugar.init Lexer_sugar.token lb in
    close_in c;
    let fd = openfile ((Filename.remove_extension file)^".choc")
        [O_CREAT; O_TRUNC; O_WRONLY] 0o755 in
    let oc = out_channel_of_descr fd in
    set_binary_mode_out oc false;
    let fmt = formatter_of_out_channel oc in
    fprintf fmt "%s@." f;
    close_out oc;
    exit 0
  with e -> close_in c; match e with
    | Parser_sugar.Error ->
      reportPos (lexeme_start_p lb, lexeme_end_p lb);
      eprintf "Syntax error@.";
      exit 1
    | Invalid_argument x ->
      reportPos (lexeme_start_p lb, lexeme_end_p lb);
      eprintf "Syntax error: %s@." x;
      exit 1
    | e ->
      eprintf "Anomaly: %s@." (Printexc.to_string e);
      exit 2
